@@ -1599,9 +1599,9 @@ procfs_attach (args, from_tty)
       exec_file = (char *) get_exec_file (0);
 
       if (exec_file)
-	printf ("Attaching program `%s', pid %d\n", exec_file, pid);
+	printf ("Attaching to program `%s', %s\n", exec_file, target_pid_to_str (pid));
       else
-	printf ("Attaching pid %d\n", pid);
+	printf ("Attaching to %s\n", target_pid_to_str (pid));
 
       fflush (stdout);
     }
@@ -1632,8 +1632,8 @@ procfs_detach (args, from_tty)
       char *exec_file = get_exec_file (0);
       if (exec_file == 0)
 	exec_file = "";
-      printf ("Detaching program: %s pid %d\n",
-	      exec_file, inferior_pid);
+      printf ("Detaching from program: %s %s\n",
+	      exec_file, target_pid_to_str (inferior_pid));
       fflush (stdout);
     }
   if (args)
@@ -1664,8 +1664,8 @@ static void
 procfs_files_info (ignore)
      struct target_ops *ignore;
 {
-  printf ("\tUsing the running image of %s process %d via /proc.\n",
-	  attach_flag? "attached": "child", inferior_pid);
+  printf ("\tUsing the running image of %s %s via /proc.\n",
+	  attach_flag? "attached": "child", target_pid_to_str (inferior_pid));
 }
 
 /* ARGSUSED */
@@ -1761,7 +1761,7 @@ do_attach (pid)
 	}
       else
 	{
-	  printf ("Ok, gdb will wait for process %u to stop.\n", pid);
+	  printf ("Ok, gdb will wait for %s to stop.\n", target_pid_to_str (pid));
 	}
     }
 
@@ -2150,50 +2150,30 @@ set_proc_siginfo (pip, signo)
     }
 }
 
-/*
-
-LOCAL FUNCTION
-
-	procfs_resume -- resume execution of the inferior process
-
-SYNOPSIS
-
-	void procfs_resume (int step, int signo)
-
-DESCRIPTION
-
-	Resume execution of the inferior process.  If STEP is nozero, then
-	just single step it.  If SIGNAL is nonzero, restart it with that
-	signal activated.
-
-NOTE
-
-	It may not be absolutely necessary to specify the PC value for
-	restarting, but to be safe we use the value that gdb considers
-	to be current.  One case where this might be necessary is if the
-	user explicitly changes the PC value that gdb considers to be
-	current.  FIXME:  Investigate if this is necessary or not.
-
-	When attaching to a child process, if we forced it to stop with
-	a PIOCSTOP, then we will have set the nopass_next_sigstop flag.
-	Upon resuming the first time after such a stop, we explicitly
-	inhibit sending it another SIGSTOP, which would be the normal
-	result of default signal handling.  One potential drawback to
-	this is that we will also ignore any attempt to by the user
-	to explicitly continue after the attach with a SIGSTOP.  Ultimately
-	this problem should be dealt with by making the routines that
-	deal with the inferior a little smarter, and possibly even allow
-	an inferior to continue running at the same time as gdb.  (FIXME?)
- */
+/* Resume execution of process PID.  If STEP is nozero, then
+   just single step it.  If SIGNAL is nonzero, restart it with that
+   signal activated.  */
 
 static void
-procfs_resume (step, signo)
+procfs_resume (pid, step, signo)
+     int pid;
      int step;
      int signo;
 {
+  int signal_to_pass;
+
   errno = 0;
   pi.prrun.pr_flags = PRSTRACE | PRSFAULT | PRCFAULT;
 
+#if 0
+  /* It should not be necessary.  If the user explicitly changes the value,
+     value_assign calls write_register_bytes, which writes it.  */
+/*	It may not be absolutely necessary to specify the PC value for
+	restarting, but to be safe we use the value that gdb considers
+	to be current.  One case where this might be necessary is if the
+	user explicitly changes the PC value that gdb considers to be
+	current.  FIXME:  Investigate if this is necessary or not.  */
+
 #ifdef PRSVADDR_BROKEN
 /* Can't do this under Solaris running on a Sparc, as there seems to be no
    place to put nPC.  In fact, if you use this, nPC seems to be set to some
@@ -2202,11 +2182,46 @@ procfs_resume (step, signo)
 
   pi.prrun.pr_vaddr = (caddr_t) *(int *) &registers[REGISTER_BYTE (PC_REGNUM)];
   pi.prrun.pr_flags != PRSVADDR;
+#endif
 #endif
 
-  if (signo && !(signo == SIGSTOP && pi.nopass_next_sigstop))
+  if (signo == SIGSTOP && pi.nopass_next_sigstop)
+    /* When attaching to a child process, if we forced it to stop with
+       a PIOCSTOP, then we will have set the nopass_next_sigstop flag.
+       Upon resuming the first time after such a stop, we explicitly
+       inhibit sending it another SIGSTOP, which would be the normal
+       result of default signal handling.  One potential drawback to
+       this is that we will also ignore any attempt to by the user
+       to explicitly continue after the attach with a SIGSTOP.  Ultimately
+       this problem should be dealt with by making the routines that
+       deal with the inferior a little smarter, and possibly even allow
+       an inferior to continue running at the same time as gdb.  (FIXME?)  */
+    signal_to_pass = 0;
+  else if (signo == SIGTSTP
+	   && pi.prstatus.pr_cursig == SIGTSTP
+	   && pi.prstatus.pr_action.sa_handler == SIG_DFL)
+
+    /* We are about to pass the inferior a SIGTSTP whose action is
+       SIG_DFL.  The SIG_DFL action for a SIGTSTP is to stop
+       (notifying the parent via wait()), and then keep going from the
+       same place when the parent is ready for you to keep going.  So
+       under the debugger, it should do nothing (as if the program had
+       been stopped and then later resumed.  Under ptrace, this
+       happens for us, but under /proc, the system obligingly stops
+       the process, and wait_for_inferior would have no way of
+       distinguishing that type of stop (which indicates that we
+       should just start it again), with a stop due to the pr_trace
+       field of the prrun_t struct.
+
+       Note that if the SIGTSTP is being caught, we *do* need to pass it,
+       because the handler needs to get executed.  */
+    signal_to_pass = 0;
+  else
+    signal_to_pass = signo;
+
+  if (signal_to_pass)
     {
-      set_proc_siginfo (&pi, signo);
+      set_proc_siginfo (&pi, signal_to_pass);
     }
   else
     {
@@ -2538,7 +2553,8 @@ info_proc_siginfo (pip, summary)
 	    }
 	  if (sip -> si_code <= 0)
 	    {
-	      printf_filtered ("sent by pid %d, uid %d ", sip -> si_pid,
+	      printf_filtered ("sent by %s, uid %d ",
+			       target_pid_to_str (sip -> si_pid),
 			       sip -> si_uid);
 	    }
 	  else
@@ -2553,8 +2569,8 @@ info_proc_siginfo (pip, summary)
 		}
 	      else if ((sip -> si_signo == SIGCHLD))
 		{
-		  printf_filtered ("child pid %u, status %u ",
-				   sip -> si_pid,
+		  printf_filtered ("child %s, status %u ",
+				   target_pid_to_str (sip -> si_pid),
 				   sip -> si_status);
 		}
 	      else if ((sip -> si_signo == SIGPOLL))
@@ -2576,7 +2592,7 @@ info_proc_siginfo (pip, summary)
 	    }
 	  if (sip -> si_code <= 0)
 	    {
-	      printf_filtered ("\t%-16u %s\n", sip -> si_pid,
+	      printf_filtered ("\t%-16u %s\n", sip -> si_pid, /* XXX need target_pid_to_str() */
 			       "PID of process sending signal");
 	      printf_filtered ("\t%-16u %s\n", sip -> si_uid,
 			       "UID of process sending signal");
@@ -2599,7 +2615,7 @@ info_proc_siginfo (pip, summary)
 		}
 	      else if ((sip -> si_signo == SIGCHLD))
 		{
-		  printf_filtered ("\t%-16u %s.\n", sip -> si_pid,
+		  printf_filtered ("\t%-16u %s.\n", sip -> si_pid, /* XXX need target_pid_to_str() */
 				   "Child process ID");
 		  printf_filtered ("\t%-16u %s.\n", sip -> si_status,
 				   "Child process exit value or signal");
@@ -2961,7 +2977,8 @@ info_proc (args, from_tty)
 
   if (!pip -> valid)
     {
-      error ("No process.  Run an inferior or specify an explicit pid.");
+      error ("\
+No process.  Start debugging a program or specify an explicit process ID.");
     }
   if (ioctl (pip -> fd, PIOCSTATUS, &(pip -> prstatus)) < 0)
     {
